@@ -1,6 +1,4 @@
-"""
-
-.. currentmodule:: lino.utils.mti
+""".. currentmodule:: lino.utils.mti
 
 Warning: This document is **deprecated**. 
 See :mod:`lino.test_apps.nomti`
@@ -257,11 +255,9 @@ The :func:`create_child` function
 
 This function is for rather internal use.
 
-:doc:`Python dumps </topics/dumpy>` 
-generated by :class:`north.dumppy.Serializer` 
-use this function
-for creating MTI children instances 
-without having to lookup their parent.
+:ref:`Python dumps <dpy>` generated by
+:class:`lino.utils.dpy.Serializer` use this function for creating MTI
+children instances without having to lookup their parent.
 
 In a Python dump we are in a special situation:
 All Place instances are being generated first,
@@ -293,7 +289,7 @@ The only thing you can do with it is to save it:
 
 The `save` and `full_clean` methods are the only methods that 
 will be called by 
-:class:`north.dumppy.Deserializer`.
+:class:`lino.utils.dpy.Deserializer`.
 
 To test whether `create_child` did her job, 
 we must re-read an instance:
@@ -343,4 +339,3 @@ Related documents
 - `Figure out child type with Django MTI or specify type as field? <http://stackoverflow.com/questions/3990470/figure-out-child-type-with-django-mti-or-specify-type-as-field>`_
 
 """
-
